@@ -1,5 +1,5 @@
 
-# $Id: helpers.R 482 2021-12-21 16:02:28Z thothorn $
+# $Id: helpers.R 486 2022-07-07 08:53:38Z thothorn $
 
 ### model.matrix.coxph doesn't return contrasts etc.
 #model.matrix.coxph <- function(object, ...) {
@@ -190,6 +190,16 @@ vcovsurvreg <- function(object, ...) {
     return(sigma[1:p, 1:p])
 }
 
+### nlme:::gls
+model.matrix.gls <- function(object, ...)
+    model.matrix(terms(object), data = nlme::getData(object), ...)
+
+model.frame.gls <- function(object, ...)
+    model.frame(formula(object), data = nlme::getData(object), ...)
+
+terms.gls <- function(object, ...)
+    terms(model.frame(object), ...)
+
 modelparm.survreg <- function(model, coef. = coef, vcov. = vcovsurvreg, df = NULL, ...)
     modelparm.default(model, coef. = coef., vcov. = vcov., df = df, ...)
 
@@ -234,6 +244,41 @@ modelparm.fixest <- function(model, coef. = coef, vcov. = vcov, df = NULL, ...)
     modelparm.default(model, coef. = coef., vcov. = vcov., df = df, ...)
 }
 
+### gamlss (donated by Marcio A Diniz)
+model.matrix.gamlss <- function(object, ...) {
+    cf <- na.exclude(coef(object))
+    
+    ### extract model matrix, frame and terms
+    mm <- model.matrix(terms(object),
+                       data = model.frame(object))
+    aux <- list(assign = attributes(mm)[["assign"]],
+                contrasts = attributes(mm)[["contrasts"]])
+    
+    mm <- mm[, 1:length(cf)]
+    attr(mm, "assign") <- aux$assign[1:length(cf)]
+    attr(mm, "contrasts") <- aux$contrasts
+    mm
+}
+
+
+gamlss.coef <- function(object, ...) {
+    dots <- list(...)
+    #class(object) <- class(object)[1]
+    cf <- na.exclude(coef(object, what = dots$what))
+    cf
+}
+
+gamlss.vcov <- function(object, ...) {
+    dots <- list(...)
+    #class(object) <- class(object)[1]
+    
+    p <- match(dots$what, object$parameters)
+    
+    vc <- vcov(object, what = dots$what) 
+    index <- which(cumsum(rownames(vc) == "(Intercept)") == p)
+    vc[index, index, drop = FALSE]
+}
+
 ### modified from package MASS  
 MPinv <- function (X, tol = sqrt(.Machine$double.eps))
 {
